﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Microsoft.SqlServer.Dts.Pipeline.Design;
using Microsoft.SqlServer.Dts.Runtime.Design;
using Microsoft.SqlServer.Dts.Pipeline;
using Microsoft.SqlServer.Dts.Runtime;
using Microsoft.SqlServer.Dts.Pipeline.Wrapper;
using Microsoft.SqlServer.Dts.Runtime.Wrapper;

using Microsoft.Ssdqs.Cleansing.Correction.Define;
using Microsoft.Ssdqs.Component.Common.Logic;
using Microsoft.Ssdqs.Component.Common.Messages;
using Microsoft.Ssdqs.Component.Common.UI.Common;
using Microsoft.Ssdqs.Component.Common.Utilities;
using Microsoft.Ssdqs.Core.Service.DataQuality;
using Microsoft.Ssdqs.Core.Service.KnowledgebaseManagement.Define;
using Microsoft.Ssdqs.Flow.DataQuality;
using Microsoft.Ssdqs.Infra.Codes;
using Microsoft.Ssdqs.Infra.Messages;
using Microsoft.Ssdqs.Infra.Utilities;
using Microsoft.Ssdqs.Matching.Define;
using Microsoft.Ssdqs.DataService.Define;
using Microsoft.Ssdqs.Proxy.EntryPoint;
using Microsoft.Ssdqs.Flow.Notification;
using Microsoft.SqlServer.MessageBox;

namespace oh22is.SqlServer.DQS
{
    public partial class frmMatchingUI : Form
    {

        #region Variables

        /// <summary>
        /// Cache the component metadata, service provider interface and
        /// connections collection so we can use it during Edit().
        /// </summary>
        private IDTSComponentMetaData100 _component;
        private IServiceProvider _serviceProvider;
        private Connections _connections;
        private Variables _variables;
        private ConnectionManager _dqsConnection = null;
        private string _serverName = null;
        private DataTable _dtInputColumns;
        private DataTable _dtDomains;
        private DataTable _dtOrigDomains;
        private long _currentSession = 0;
        private string _databaseName = "DQS_MAIN";
        private string _knowledgeBase = null;
        private bool _loadProperties = false;
        private BindingSource bs;
     
        /// <summary>
        /// COMPONENT PROPERTIES
        /// </summary>
        private string DQS_CONNECTION_MANAGER = null;
        private string DQS_KNOWLEDGE_BASE = null;
        private string MAPPING_VALUES = null;
        
        /// <summary>
        /// ADVANCED PROPERTIES
        /// </summary>
        private bool ENCRYPT_CONNECTION = false;
        private bool CLEANUP_DQS_PROJECTS = false;
        private int SCORE_VALUE = 80;

        private const string FIELD_NOT_MAPPED = "Field not mapped.";
        private const string NO_CUSTOM_ORDER = "<No Custom Ordering>";

        #endregion

        #region localProperties

        /// <summary>
        /// 
        /// </summary>
        private Knowledgebase SelectedKnowledgeBase
        {
            get
            {
                Knowledgebase KB = null;
                var knowledgeBase = ProxyEntryPointFactory.GetKnowledgebaseManagementEntryPoint(_serverName, _databaseName, _currentSession);
                {
                    var allKbs = knowledgeBase.KnowledgebaseGet();
                    foreach (Knowledgebase kb in allKbs)
                    {
                        if (kb.Name == _knowledgeBase)
                        {
                            KB = kb;
                            break;
                        }
                    }
                }
                return KB;
            }
        }

        #endregion

        #region UI Functions

        /// <summary>
        /// 
        /// </summary>
        /// <param name="dtsComponentMetadata"></param>
        /// <param name="serviceProvider"></param>
        /// <param name="variables"></param>
        /// <param name="connections"></param>
        public frmMatchingUI(IDTSComponentMetaData100 dtsComponentMetadata, IServiceProvider serviceProvider, Variables variables, Connections connections)
        {
            InitializeComponent();
            _connections = connections;
            _component = dtsComponentMetadata;
            _serviceProvider = serviceProvider;
            _variables = variables;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void frmMatchingUI_Load(object sender, EventArgs e)
        {

            _loadProperties = true;
            _dtInputColumns = new DataTable("InputColumns");
            _dtDomains = new DataTable("Domains");
            populateComponentProperties();
            _loadProperties = false;

            dgvInputColumns.Columns[1].ReadOnly = true;
            dgvInputColumns.Columns[2].ReadOnly = true;

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOK_Click(object sender, EventArgs e)
        {
            if (AreAllDomainsMappedCorrectly())
            {
                SaveMappingValues();

                if (cbDQKnowledgeBase.SelectedIndex != -1)
                {
                    _component.CustomPropertyCollection["DqsKnowledgeBase"].Value = cbDQKnowledgeBase.Items[cbDQKnowledgeBase.SelectedIndex];
                }
                else
                {
                    _component.CustomPropertyCollection["DqsKnowledgeBase"].Value = null;
                }

                _component.CustomPropertyCollection["ScoreValue"].Value = trackBar.Value.ToString();
                _component.CustomPropertyCollection["EncryptConnection"].Value = cbEncryptConnection.Checked.ToString();
                _component.CustomPropertyCollection["MappingValues"].Value = MAPPING_VALUES;
                _component.CustomPropertyCollection["CleanUpDqsProjects"].Value = cbCleanDQSProjects.Checked.ToString();

                this.DialogResult = System.Windows.Forms.DialogResult.OK;
            }
            else
            {
                MessageBox.Show("Not all domains have been assigned or have been assigned twice. Please assign all domains properly.", "SSIS DQS Matching Transformation", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

        /// <summary>
        /// Creates a new DQS connection manager
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnNew_Click(object sender, EventArgs e)
        {
            try
            {

                /// Creates a new Connection Service and opens the DQS Cleansing Connection Manager window
                IDtsConnectionService connService = (IDtsConnectionService)_serviceProvider.GetService(typeof(IDtsConnectionService));
                System.Collections.ArrayList created = connService.CreateConnection("DQS");

                foreach (ConnectionManager cm in created)
                {
                    cbDQConnectionManager.Items.Insert(0, cm.Name);
                }

                if (created.Count > 0)
                {
                    cbDQConnectionManager.SelectedIndex = 0;
                }
                else
                {
                    cbDQConnectionManager.SelectedIndex = -1;
                }
            }
            catch(Exception ex)
            {
                ExceptionMessageBox("An error occurred while creating a connection manager.",ex);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbDQConnectionManager_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                string name = cbDQConnectionManager.Items[cbDQConnectionManager.SelectedIndex].ToString();

                foreach (ConnectionManager cm in _connections)
                {
                    if (cm.Name == name)
                    {
                        _dqsConnection = cm;
                        _component.RuntimeConnectionCollection["DQSConnectionManager"].ConnectionManager = DtsConvert.GetExtendedInterface(_connections[cbDQConnectionManager.SelectedItem]);
                        _component.RuntimeConnectionCollection["DQSConnectionManager"].ConnectionManagerID = _connections[cbDQConnectionManager.SelectedItem].ID;
                        break;
                    }
                }

                if (_dqsConnection != null)
                {

                    InitializeProxy();

                    _currentSession = CreateClientSession(_dqsConnection, _databaseName);                   

                    Dictionary<string, string> connectionStringParameters = DataQualityConnectorFinals.GetConnectionStringParameters(_dqsConnection.ConnectionString);
                    _serverName = connectionStringParameters["ServerName"];

                    var knowledgeBase = ProxyEntryPointFactory.GetKnowledgebaseManagementEntryPoint(_serverName, _databaseName, _currentSession);
                    {
                        var allKbs = knowledgeBase.KnowledgebaseGet();
                        cbDQKnowledgeBase.Items.Clear();
                        foreach (Knowledgebase kb in allKbs)
                        {
                            this.cbDQKnowledgeBase.Items.Add(kb.Name);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                ExceptionMessageBox("An error occurred while retrieving the connection manager.",ex);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void cbDQKnowledgeBase_SelectedIndexChanged(object sender, EventArgs e)
        {

            try
            {
                if (!_loadProperties) SaveMappingValues();

                _knowledgeBase = cbDQKnowledgeBase.Items[cbDQKnowledgeBase.SelectedIndex].ToString();
                tvMatchingRules.Nodes.Clear();
                _dtDomains.Clear();

                if (_dqsConnection != null)
                {

                    InitializeProxy(); 

                    var matchingRules = ProxyEntryPointFactory.GetMatchingEntryPoint(_serverName, _databaseName, _currentSession, SelectedKnowledgeBase.Id);
                    {
                        var allMRules = matchingRules.MatchingRulesGet();
                        if (allMRules != null)
                        {
                            var metadata = ProxyEntryPointFactory.GetMetadataManagementEntryPoint(_serverName, _databaseName, _currentSession, SelectedKnowledgeBase.Id);
                            int i = 0;

                            foreach (MatchingRule mr in allMRules)
                            {
                               TreeNode node = new TreeNode(mr.Name);
                               node.Tag = mr.Id;
                               node.ImageIndex = 0;

                               tvMatchingRules.Nodes.Add(node);

                               var colCompositeDomain = metadata.CompositeDomainGetAll();
                                
                               if (colCompositeDomain != null)
                                {
                                    foreach (CompositeDomain cd in colCompositeDomain)
                                    {

                                        if (AreAllCompositeDomainElementsInMatchingRule(cd.ChildDomainIds.ToArray(), mr))
                                        {
                                            string compositeDomainName = cd.Name;
                                            TreeNode compositeDomainNode = new TreeNode(compositeDomainName);
                                            compositeDomainNode.Tag = cd.Id;
                                            compositeDomainNode.ImageIndex = 2;
                                            compositeDomainNode.SelectedImageIndex = 2;
                                            tvMatchingRules.Nodes[i].Nodes.Add(compositeDomainNode);
                                            
                                            foreach (long domainId in cd.ChildDomainIds.ToArray())
                                            {
                                                string DomainName = metadata.DomainGetById(domainId).Name.ToString();
                                                TreeNode DomainNode = new TreeNode(DomainName);
                                                DomainNode.Tag = domainId;
                                                DomainNode.ImageIndex = 1;
                                                DomainNode.SelectedImageIndex = 1;
                                                compositeDomainNode.Nodes.Add(DomainNode);
                                                
                                            }

                                            AddDomainToDataTable(cd.Id, cd.Name, true);
                                        }
                                    }
                                }

                                foreach (MatchingRuleDomainElement domain in mr.DomainElements)
                                {
                                    string domainName;
                                    var dom = metadata.DomainGetById(domain.DomainId);

                                    if (dom == null)
                                    {
                                        CompositeDomain cd = metadata.CompositeDomainGetById(domain.DomainId);
                                        domainName = cd.Name;
                                        string compositeDomainName = cd.Name;
                                        TreeNode compositeDomainNode = new TreeNode(compositeDomainName);
                                        compositeDomainNode.Tag = cd.Id;
                                        compositeDomainNode.ImageIndex = 2;
                                        compositeDomainNode.SelectedImageIndex = 2;
                                        tvMatchingRules.Nodes[i].Nodes.Add(compositeDomainNode);

                                        AddDomainToDataTable(domain.DomainId, domainName, true);

                                        foreach (long domainId in cd.ChildDomainIds.ToArray())
                                        {
                                            string CD_DomainName = metadata.DomainGetById(domainId).Name.ToString();
                                            TreeNode DomainNode = new TreeNode(CD_DomainName);
                                            DomainNode.Tag = domainId;
                                            DomainNode.ImageIndex = 1;
                                            DomainNode.SelectedImageIndex = 1;
                                            compositeDomainNode.Nodes.Add(DomainNode);

                                            AddDomainToDataTable(domainId, CD_DomainName, false);

                                        }
                                    }
                                    else
                                    {
                                        domainName = metadata.DomainGetById(domain.DomainId).Name;
                                        TreeNode domainNode = new TreeNode(domainName);
                                        domainNode.Tag = domain.DomainId;
                                        domainNode.ImageIndex = 1;
                                        domainNode.SelectedImageIndex = 1;
                                        tvMatchingRules.Nodes[i].Nodes.Add(domainNode);

                                        AddDomainToDataTable(domain.DomainId, domainName, false);
                                    }
                                    
                                }

                                tvMatchingRules.Nodes[i].Expand();

                                i++;
                            }
                        }
                    }
                }

                if (!_loadProperties) RestoreMappingValues();
                _dtOrigDomains = _dtDomains.Copy();
            }
            catch (Exception ex)
            {
                ExceptionMessageBox("The Knowledge Base could not be loaded." + Environment.NewLine + "Please check the Knowledge Base with the DQS Client.", ex);
            }

        }

        /// <summary>
        /// Sets the SCORE_VALUE for the matching output
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void trackBar_Scroll_1(object sender, EventArgs e)
        {
            lblScoreValue.Text = String.Format("{0} %", trackBar.Value.ToString());
            SCORE_VALUE = trackBar.Value;
        }

        /// <summary>
        /// Adds or deletes an input column from the mapping data-grid
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvInputColumns_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            try
            {
                if (e.ColumnIndex == 0)
                {
                    bool check = !(bool)dgvInputColumns[0, e.RowIndex].Value;
                    if (check)
                    {
                        dgvInputColumns[2, e.RowIndex].Value = String.Empty;

                        bool rowExist = false;
                        foreach (DataGridViewRow row in dgvMapping.Rows)
                        {
                            if (row.Cells[0].Value.ToString() == dgvInputColumns[1, e.RowIndex].Value.ToString())
                            {
                                rowExist = true;
                            }
                        }

                        if (!rowExist)
                        {
                            dgvMapping.Rows.Add();
                            int newRow = dgvMapping.Rows.GetLastRow(new DataGridViewElementStates());
                            dgvMapping[0, newRow].Value = dgvInputColumns[1, e.RowIndex].Value;

                            Hashtable SelectedDomains = GetAllSelectedDomain();

                            DataRow[] row = _dtDomains.Select(String.Format("Domain = '{0}'", dgvInputColumns[1, e.RowIndex].Value.ToString()));
                            if (row.Count() > 0)
                            {
                                if (!SelectedDomains.ContainsValue(dgvInputColumns[1, e.RowIndex].Value.ToString()))
                                {
                                   dgvMapping[1, newRow].Value = row[0][1].ToString();
                                }
                            }
                        }
                    }
                    else
                    {
                        dgvInputColumns[2, e.RowIndex].Value = FIELD_NOT_MAPPED;
                        foreach (DataGridViewRow row in dgvMapping.Rows)
                        {
                            if (row.Cells[0].Value.ToString() == dgvInputColumns[1, e.RowIndex].Value.ToString())
                            {
                                dgvMapping.Rows.Remove(row);
                                break;
                            }
                        }
                    }

                    dgvInputColumns[0, e.RowIndex].Value = check;
                    dgvInputColumns.UpdateCellValue(0, e.RowIndex);

                }
            }
            catch (Exception ex)
            {
                ExceptionMessageBox("Sorry, during mapping columns to domains, an error has occurred...", ex);
            }

        }

        /// <summary>
        /// Opens the project website on codeplex
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkCodeplex_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://ssisdqsmatching.codeplex.com");
        }

        /// <summary>
        /// Opens the website of oh22information services
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkOH22_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.oh22.is");
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnHelp_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("https://ssisdqsmatching.codeplex.com/documentation");
        }

        #endregion

        #region DataGridView

        /// <summary>
        /// Creates the Mapping Data Grid view
        /// </summary>
        private void CreateMappingColumns()
        {

            DataGridViewTextBoxCell cellInput = new DataGridViewTextBoxCell();

            DataGridViewComboBoxCell cellDomains = new DataGridViewComboBoxCell();
            cellDomains.DataSource = _dtDomains;
            cellDomains.DisplayMember = "Domain";
            cellDomains.ValueMember = "Domain";

            bs = new BindingSource();
            DataView dv = new DataView(_dtDomains);
            bs.DataSource = dv;

            DataGridViewColumn colInput = new DataGridViewColumn();
            colInput.CellTemplate = cellInput;
            colInput.HeaderText = "Input Columns";
            colInput.ReadOnly = true;

            DataGridViewColumn colDomain = new DataGridViewColumn();
            colDomain.CellTemplate = cellDomains;
            colDomain.HeaderText = "Domains";

            dgvMapping.Columns.Add(colInput);
            dgvMapping.Columns.Add(colDomain);

            dgvMapping.CellBeginEdit += dgvMapping_CellBeginEdit;
            dgvMapping.CellEndEdit += dgvMapping_CellEndEdit;

            dgvMapping.DataError += dgvMapping_DataError;

        }

        #endregion

        #region DQS

        /// <summary>
        /// checks whether the connection manager is a DQS connection manager or not
        /// </summary>
        /// <param name="cm"></param>
        /// <returns></returns>
        private bool isDQSConnection(ConnectionManager cm)
        {
            return cm.CreationName.StartsWith("DQS");
        }

        /// <summary>
        /// Creates a session with the DQS server and returns the current session id
        /// </summary>
        /// <param name="connectionManager">The DQS connection manager</param>
        /// <param name="databaseName">The DQS database name</param>
        /// <returns>The current session id</returns>
        private long CreateClientSession(ConnectionManager connectionManager, string databaseName)
        {

            if (connectionManager != null)
            {
                if (databaseName != null)
                {
                    Dictionary<string, string> connectionStringParameters = DataQualityConnectorFinals.GetConnectionStringParameters(connectionManager.ConnectionString);
                    NotificationEntryPointClient notificationEntryPoint = ProxyEntryPointFactory.GetNotificationEntryPoint(connectionStringParameters["ServerName"], databaseName);
                    NotificationSessionInfo notificationSessionInfo = notificationEntryPoint.NotificationSessionCreate(ClientType.SsisClient);
                    return notificationSessionInfo.SessionId;
                }
                else
                {
                    throw new ArgumentNullException("databaseName");
                }
            }
            else
            {
                throw new ArgumentNullException("connectionManager");
            }

        }

        /// <summary>
        /// Checks if a Composite Domain is part of Matching Rule
        /// </summary>
        /// <param name="domainId"></param>
        /// <param name="matchingRule"></param>
        /// <returns></returns>
        private bool AreAllCompositeDomainElementsInMatchingRule(long[] domainId, MatchingRule matchingRule)
        {
            bool retValue = true;
            MatchingRuleDomainElement[] DomainElements = new MatchingRuleDomainElement[matchingRule.DomainElements.Count()];
            matchingRule.DomainElements.CopyTo(DomainElements, 0);
            

            for (int cdI = 0; cdI < domainId.Count(); cdI++)
            {
                var result = Array.Find(DomainElements, element => element.DomainId == domainId[cdI]);
                if (result == null) return false;

            }

            return retValue;
        }

        /// <summary>
        /// Returns the ID of a knowledge base based on the name 
        /// </summary>
        /// <param name="knowledgeBaseName"></param>
        /// <returns></returns>
        private long KBGetIdByName(string knowledgeBaseName)
        {
            long ID = 0;

            var knowledgeBase = ProxyEntryPointFactory.GetKnowledgebaseManagementEntryPoint(_serverName, _databaseName, _currentSession);
            {

                cbDQKnowledgeBase.Items.Clear();
                Knowledgebase KB = Array.Find(knowledgeBase.KnowledgebaseGet().ToArray(), element => element.Name == knowledgeBaseName);
                ID = KB.Id;

            }

            return ID;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private bool AreAllDomainsMappedCorrectly()
        {
            bool retValue = true;
            ArrayList selectedDomains = new ArrayList();

            var metadata = ProxyEntryPointFactory.GetMetadataManagementEntryPoint(_serverName, _databaseName, _currentSession, SelectedKnowledgeBase.Id);
            
            for (int i = 0; i < dgvMapping.RowCount; i++)
            {
                if (dgvMapping.Rows[i].Cells[1].Value == null) return false;
                string domain = dgvMapping.Rows[i].Cells[1].Value.ToString();
                if (domain != null)
                {
                  CompositeDomain cd = metadata.CompositeDomainGetByName(dgvMapping.Rows[i].Cells[1].Value.ToString());
                  if (cd == null)                
                  {
                      if (selectedDomains.IndexOf(domain) > -1) return false;
                      selectedDomains.Add(domain);
                  }
                  else
                  {
                    foreach(var dom in cd.ChildDomainIds)
                    {
                        if (selectedDomains.IndexOf(metadata.DomainGetById(dom)) >  -1) return false;
                        selectedDomains.Add(metadata.DomainGetById(dom).Name);
                    }
                  }
              }
            }

            using (var matchingRules = ProxyEntryPointFactory.GetMatchingEntryPoint(_serverName, _databaseName, _currentSession, SelectedKnowledgeBase.Id))
            {
                var allMRules = matchingRules.MatchingRulesGet();
                if (allMRules != null)
                {
                    foreach (MatchingRule mr in allMRules)
                    {
                        foreach(var dom in mr.DomainElements)
                        {
                            var domain = metadata.DomainGetById(dom.DomainId);
                            if (domain != null)
                            {
                                if (selectedDomains.IndexOf(domain.Name) == -1) return false;
                            }
                            else
                            {
                                foreach (long id in metadata.CompositeDomainGetById(dom.DomainId).ChildDomainIds)
                                {
                                    if (selectedDomains.IndexOf(metadata.DomainGetById(id).Name) == -1) return false; 
                                }
                            }
                        }
                    }

                }
            }

            return retValue;
        }

        #endregion

        /// <summary>
        /// Loads the UI components in the form with appropriate values.
        /// </summary>
        private void populateComponentProperties()
        {

            try
            {
                if (_component.RuntimeConnectionCollection.Count > 0)
                {
                    IDTSRuntimeConnection100 conn = _component.RuntimeConnectionCollection[0];
                    if (conn != null
                        && conn.ConnectionManagerID.Length > 0
                        && _connections.Contains(conn.ConnectionManagerID))
                    {
                        DQS_CONNECTION_MANAGER = _connections[conn.ConnectionManagerID].Name;
                    }
                }
            }
            catch (Exception ex)
            {
                ExceptionMessageBox("An error occurred while retrieving the connection manager.",ex);
            }

            DQS_KNOWLEDGE_BASE = (_component.CustomPropertyCollection["DqsKnowledgeBase"].Value != null ? _component.CustomPropertyCollection["DqsKnowledgeBase"].Value.ToString() :String.Empty);
            ENCRYPT_CONNECTION = (_component.CustomPropertyCollection["EncryptConnection"].Value != null ? Convert.ToBoolean(_component.CustomPropertyCollection["EncryptConnection"].Value.ToString()) : false);
            MAPPING_VALUES = (_component.CustomPropertyCollection["MappingValues"].Value != null ? _component.CustomPropertyCollection["MappingValues"].Value.ToString() :String.Empty);
            CLEANUP_DQS_PROJECTS = (_component.CustomPropertyCollection["CleanUpDqsProjects"].Value != null ? Convert.ToBoolean(_component.CustomPropertyCollection["CleanUpDqsProjects"].Value.ToString()) : false);
            
            bool DQCMExist = false;

            this.cbDQConnectionManager.Items.Clear();
            foreach (ConnectionManager cm in _connections)
            {
                if (isDQSConnection(cm))
                {
                    this.cbDQConnectionManager.Items.Add(cm.Name);
                    if (cm.Name == DQS_CONNECTION_MANAGER) DQCMExist = true;
                }
            }

            if (DQCMExist)
            {
                cbDQConnectionManager.SelectedItem = cbDQConnectionManager.Items[cbDQConnectionManager.Items.IndexOf(DQS_CONNECTION_MANAGER)];
            }

            //Fill a DataTable with Input Columns
            populateInputDataTable();

            ///Create a DataTable for domain vlaues
            createDomainTable();

            ///Create columns as templates for mapping DataGridView
            CreateMappingColumns();

            ///Create values for Input Column DataGridView
            CreateInputColumns();

            DataTable SortTable = _dtInputColumns.Copy();
            DataRow doNotSortRow = SortTable.NewRow();
            doNotSortRow[0] = false;
            doNotSortRow[1] = NO_CUSTOM_ORDER;
            SortTable.Rows.Add(doNotSortRow);

            if (_component.CustomPropertyCollection["ScoreValue"].Value != null)
            {
                int trackBarValue = Convert.ToInt32(_component.CustomPropertyCollection["ScoreValue"].Value.ToString());
                if (trackBarValue < 80 || trackBarValue > 100)
                {
                    trackBar.Value = 80;
                }
                else
                {
                    trackBar.Value = trackBarValue;
                }
            }
            else
            {
                trackBar.Value = 80;
            }
            
            lblScoreValue.Text = String.Format("{0} %", trackBar.Value.ToString());

            try
            {
                cbDQKnowledgeBase.SelectedItem = cbDQKnowledgeBase.Items[cbDQKnowledgeBase.Items.IndexOf(DQS_KNOWLEDGE_BASE)];
            }
            catch { }

            cbEncryptConnection.Checked = ENCRYPT_CONNECTION;
            cbCleanDQSProjects.Checked = CLEANUP_DQS_PROJECTS;

            if (!String.IsNullOrEmpty(MAPPING_VALUES))
            {
                RestoreMappingValues();
            }
                      
        }

        /// <summary>
        /// Creates the _dtDomain data table
        /// </summary>
        private void createDomainTable()
        {
            DataColumn domainID = new DataColumn("ID", System.Type.GetType("System.Int32"));
            _dtDomains.Columns.Add(domainID);

            DataColumn domainColumns = new DataColumn("Domain", System.Type.GetType("System.String"));
            _dtDomains.Columns.Add(domainColumns);

            DataColumn IsCompositeDOmain = new DataColumn("IsCompositeDomain", System.Type.GetType("System.Boolean"));
            _dtDomains.Columns.Add(IsCompositeDOmain);          

        }

        /// <summary>
        /// Creates the Iinput column datagrid
        /// </summary>
        private void CreateInputColumns()
        {
            dgvInputColumns.DataSource = _dtInputColumns.Copy();
            dgvInputColumns.Columns[0].HeaderText = "";
            dgvInputColumns.Columns[0].AutoSizeMode = DataGridViewAutoSizeColumnMode.None;
            dgvInputColumns.Columns[0].Width = 50;
            dgvInputColumns.Columns[1].HeaderText = "Input Columns";
            dgvInputColumns.Columns[2].HeaderText = "Mapping Info";
        }

        /// <summary>
        /// Creates a data table of all input columns and bind the data table to the input data grid.
        /// </summary>
        private void populateInputDataTable()
        {
            _dtInputColumns.Clear();

            DataColumn ActiveColumn = new DataColumn("ActiveColumns", System.Type.GetType("System.Boolean"));
            _dtInputColumns.Columns.Add(ActiveColumn);

            DataColumn InputColumn = new DataColumn("InputColumns", System.Type.GetType("System.String"));
            _dtInputColumns.Columns.Add(InputColumn);

            DataColumn MappingInfoColumn = new DataColumn("Info", System.Type.GetType("System.String"));
            _dtInputColumns.Columns.Add(MappingInfoColumn);

            string[] inputColumns = (MAPPING_VALUES != null ? MAPPING_VALUES.Split(';') : null);

            for (int i = 0; i < _component.InputCollection.Count; i++)
            {
                _component.InputCollection[i].InputColumnCollection.RemoveAll();
                IDTSVirtualInput100 input = _component.InputCollection[i].GetVirtualInput();
                foreach (IDTSVirtualInputColumn100 vcol in input.VirtualInputColumnCollection)
                {
                    DataRow row = _dtInputColumns.NewRow();
                    bool activeColumn = false;

                    if (inputColumns != null)
                    { 
                        foreach (string str in inputColumns)
                        {
                            if (str.Split('|')[0] == vcol.Name && !String.IsNullOrEmpty(str.Split('|')[1])) activeColumn = true;
                        }
                    }
                    
                    row[0] = activeColumn;
                    row[1] = vcol.Name;

                    if (activeColumn == false)
                    {
                        row[2] = FIELD_NOT_MAPPED;
                    }
                    else
                    {
                        row[2] = string.Empty;
                    }

                    _dtInputColumns.Rows.Add(row);
                }
            }
        }

        /// <summary>
        /// Adds a domain to the domain data table.
        /// The domain data table is used for the drop down list in the domain mapping.
        /// </summary>
        /// <param name="Id">The Id of the domain</param>
        /// <param name="Name">The Name of the domain</param>
        /// <param name="IsCompositeDomain">Indicates if the domain is a composite domain or not</param>
        private void AddDomainToDataTable(long Id, string Name, bool IsCompositeDomain)
        {
            if (_dtDomains.Select(String.Format("Id = {0}", Id)).Count() == 0)
            {
                DataRow row = _dtDomains.NewRow();
                row[0] = Id;
                row[1] = Name;
                row[2] = IsCompositeDomain;
                _dtDomains.Rows.Add(row);
            }

        }
        
        /// <summary>
        /// All mapped input columns and domains will saved as a pair
        /// ==> INPUT_COLUMN | DOMAIN
        /// Different mappings are divided by a semicolon
        /// </summary>
        private void SaveMappingValues()
        {
            MAPPING_VALUES = null;

            if (dgvMapping.RowCount > 0)
            {
                string input = null;

                foreach (DataRow row in _dtInputColumns.Rows)
                {
                    input = row[1].ToString();

                    for (int i = 0; i < dgvMapping.RowCount; i++)
                    {
                        if (input == dgvMapping.Rows[i].Cells[0].Value.ToString())
                        {
                            if (dgvMapping.Rows[i].Cells[1].Value != null)
                            {
                                MAPPING_VALUES += String.Format("{0}|{1};", input, dgvMapping.Rows[i].Cells[1].Value.ToString());
                                break;
                            }
                        }
                    }
                }

                if (!String.IsNullOrEmpty(MAPPING_VALUES)) MAPPING_VALUES = MAPPING_VALUES.Substring(0, (MAPPING_VALUES.Length - 1));

            }

        }

        /// <summary>
        /// Restores the mapping value
        /// Adds the input columns to the datagrid and map them to the domains
        /// </summary>
        private void RestoreMappingValues()
        {
          
            dgvMapping.Rows.Clear();

            foreach (DataGridViewRow dgvRow in dgvInputColumns.Rows)
            {
                dgvRow.Cells[0].Value = false;
                dgvRow.Cells[2].Value = FIELD_NOT_MAPPED;
            }

            if (MAPPING_VALUES != null)
            {
                string[] MappingRows = MAPPING_VALUES.Split(';');
                for (int i = 0; i < MappingRows.Count(); i++)
                {

                    bool SetValue = false;
                    string[] MappingColumns = MappingRows[i].Split('|');

                    if (!String.IsNullOrEmpty(MappingColumns[1])
                        && !String.IsNullOrEmpty(MappingColumns[0])
                        && _dtDomains.Select(String.Format("Domain = '{0}'", MappingColumns[1])).Count() > 0
                        && _dtInputColumns.Select(String.Format("InputColumns = '{0}'", MappingColumns[0])).Count() > 0)
                    {

                        dgvMapping.Rows.Add();
                        int lastRow = dgvMapping.Rows.GetLastRow(new DataGridViewElementStates());

                        if (_dtDomains.Select(String.Format("Domain = '{0}'", MappingColumns[1])).Count() > 0)
                        {
                            dgvMapping.Rows[lastRow].Cells[1].Value = MappingColumns[1];
                            SetValue = true;
                        }

                        if (_dtInputColumns.Select(String.Format("InputColumns = '{0}'", MappingColumns[0])).Count() > 0)
                        {
                            dgvMapping.Rows[lastRow].Cells[0].Value = MappingColumns[0];
                            SetValue = true;
                        }

                        foreach (DataGridViewRow dgvRow in dgvInputColumns.Rows)
                        {
                            if (dgvRow.Cells[1].Value.ToString() == MappingColumns[0])
                            {
                                dgvRow.Cells[0].Value = true;
                                dgvRow.Cells[2].Value = "";
                            }
                        }

                        if (!SetValue) dgvMapping.Rows.RemoveAt(dgvMapping.Rows.Count - 1);

                    }
                    else
                    {
                        foreach (DataGridViewRow dgvRow in dgvInputColumns.Rows)
                        {
                            if (dgvRow.Cells[1].Value.ToString() == MappingColumns[0])
                            {
                                dgvRow.Cells[0].Value = false;
                                dgvRow.Cells[2].Value = FIELD_NOT_MAPPED;
                            }
                        }
                    }
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="connName"></param>
        /// <returns></returns>
        private IDTSRuntimeConnection100 getRuntimeConn(string connName)
        {
            try
            {
                IDTSRuntimeConnection100 conn = _component.RuntimeConnectionCollection[connName];
                return conn;
            }
            catch
            {
                return null;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public void InitializeProxy()
        {
            try
            {
                ProxyInit.UseEncryption = cbEncryptConnection.Checked;
                if (!ProxyInit.IsInit)
                {
                    ProxyInit.Init(new SsisProxyInitParameters());
                }
                
            }
            catch (Exception ex)
            {
                ExceptionMessageBox("The Proxy could not be initialized.", ex);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="userFriendlyMessage"></param>
        /// <param name="ex"></param>
        public void ExceptionMessageBox(string userFriendlyMessage, Exception ex)
        {
            ApplicationException exTop = new ApplicationException(userFriendlyMessage, ex);
            exTop.Source = this.Text;
            ExceptionMessageBox box = new ExceptionMessageBox(exTop, ExceptionMessageBoxButtons.OK, ExceptionMessageBoxSymbol.Error);
            box.Show(this);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvMapping_CellBeginEdit(object sender, DataGridViewCellCancelEventArgs e)
        {
            if (e.ColumnIndex == 1)
            {
                // Set the combobox cell datasource to the filtered BindingSource 
                DataGridViewComboBoxCell dgcb = (DataGridViewComboBoxCell)dgvMapping[e.ColumnIndex, e.RowIndex];
                dgcb.DataSource = bs;
                string strFilter = "";
                Hashtable ht = GetAllSelectedDomain();

                string selectedValue = (dgvMapping.Rows[e.RowIndex].Cells[1].Value == null ? "" : dgvMapping.Rows[e.RowIndex].Cells[1].Value.ToString());

                foreach (var val in ht.Values)
                {
                    if (val.ToString() != selectedValue)
                    {
                        strFilter += String.Format("'{0}',", val.ToString());
                    }
                }

                if (!(strFilter.Trim().Length == 0))
                {
                    strFilter = strFilter.Substring(0, strFilter.Length - 1);
                    this.bs.Filter = "Domain not in (" + strFilter + ")";
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void dgvMapping_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            if (e.ColumnIndex == 1)
            {
                // Reset combobox cell to the unfiltered BindingSource 
                DataGridViewComboBoxCell dgcb = (DataGridViewComboBoxCell)dgvMapping[e.ColumnIndex, e.RowIndex];
                dgcb.DataSource = _dtDomains;
                this.bs.RemoveFilter();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private Hashtable GetAllSelectedDomain()
        {
            Hashtable hashTable = new Hashtable();

            using (var metadata = ProxyEntryPointFactory.GetMetadataManagementEntryPoint(_serverName, _databaseName, _currentSession, SelectedKnowledgeBase.Id))
            {
                for (int i = 0; i < dgvMapping.Rows.Count; i++)
                {
                    if (dgvMapping.Rows[i].Cells[1].Value != null)
                    {
                        string selectFilter = String.Format("Domain = '{0}'", dgvMapping.Rows[i].Cells[1].Value.ToString());
                        DataRow[] row = _dtDomains.Select(selectFilter);
                        if (row.Count() > 0)
                        {
                            if (Convert.ToBoolean(row[0][2]) == false)
                            {
                                if (!hashTable.ContainsKey(row[0][0]))
                                {
                                    hashTable.Add(row[0][0], row[0][1]);
                                }
                            }
                            else
                            {
                                if (!hashTable.ContainsKey((long)Convert.ToDouble(row[0][0])))
                                {
                                    hashTable.Add((long)Convert.ToDouble(row[0][0]), row[0][1]);
                                }

                                long[] childDomainIds = metadata.CompositeDomainGetById((long)Convert.ToDouble(row[0][0])).ChildDomainIds.ToArray();
                                for (int m = 0; m < childDomainIds.Count(); m++)
                                {
                                    if (!hashTable.ContainsKey(childDomainIds[m]))
                                    {
                                        hashTable.Add(childDomainIds[m], metadata.DomainGetById(childDomainIds[m]).Name);
                                    }
                                }
                            }

                            var compositeDomain = metadata.CompositeDomainGetAll();
                            if (compositeDomain != null)
                            {
                                foreach (CompositeDomain CD in compositeDomain)
                                {
                                    var result = Array.Find(CD.ChildDomainIds.ToArray(), element => element == (long)Convert.ToDouble(row[0][0]));
                                    if (result != 0)
                                    {
                                        if (!hashTable.ContainsKey(CD.Id))
                                        {
                                            hashTable.Add(CD.Id, CD.Name);
                                        }
                                    }
                                }
                            }

                        }
                    }
                }
            }

            return hashTable;

        }

        private void dgvMapping_DataError(object sender, DataGridViewDataErrorEventArgs anError) {}

    }
}
